<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

// Подключаем библиотеку от RetailCRM
require_once(dirname(__FILE__) . '/vendor/autoload.php');

/**
* Адаптер для коммуникации с retailCRM.
* 
* @author Maxim Zasorin, KAD Systems (©) 2017
* @date 07-12-2017
* Начало >>
*/
class Kad_Retailcrm_Client
{
	/**
	 * Объект. 
	 *
	 * @var mixed
	 */
	static private $_instance = NULL;

	/**
	 * Клиент для коммуникации с retailCRM.
	 *
	 * @var \RetailCrm\ApiClient
	 */
	protected $_client;

	/**
	 * Менеджер по-умолчанию.
	 *
	 * @var integer
	 */
	protected $_defaultManager;

	/**
	 * Способ заказа по-умолчанию.
	 *
	 * @var string
	 */
	protected $_defaultOrderMethod;

	/**
	 * Статус заказа для подписки на товар.
	 *
	 * @var string
	 */
	protected $_subscriptionOrderStatus;

	/**
	 * Создает объект.
	 *
	 * @return object
	 */
	static public function instance()
	{
		if (!isset(self::$_instance))
		{
			$aConfig = Core_Config::instance()->get('kad_retailcrm_config', array()) + array(
				'url' => '',
				'api_key' => '',
				'manager_id' => '',
				'order_type' => '',
			);

			self::$_instance = new self($aConfig['url'], $aConfig['api_key']);
			self::$_instance
				->defaultManager($aConfig['manager_id'])
				->defaultOrderMethod($aConfig['order_type'])
				->subscriptionOrderStatus($aConfig['subscription_order_status']);
		}
		return self::$_instance;
	}

	/**
	 * @param  string  $message
	 * @param  integer  $status
	 * @return void
	 */
	static public function log($message, $status = 0)
	{
		Core_Log::instance()
			->clear()
			->status($status)
			->write($message);
	}

	/**
	 * Менеджер по-умолчанию.
	 *
	 * @param  integer  $defaultManager
	 * @return self
	 */
	public function defaultManager($defaultManager)
	{
		$this->_defaultManager = $defaultManager;

		return $this;
	}

	/**
	 * Способ заказа по-умолчанию.
	 *
	 * @param  string  $defaultOrderMethod
	 * @return self
	 */
	public function defaultOrderMethod($defaultOrderMethod)
	{
		$this->_defaultOrderMethod = $defaultOrderMethod;

		return $this;
	}

	/**
	 * Статус заказа для подписки на товар.
	 *
	 * @param  string  $subscriptionOrderStatus
	 * @return self
	 */
	public function subscriptionOrderStatus($subscriptionOrderStatus)
	{
		$this->_subscriptionOrderStatus = $subscriptionOrderStatus;

		return $this;
	}

	/**
	 * Создает заказ.
	 *
	 * @param  Shop_Order_Model  $oShopOrder
	 * @return void
	 */
	public function createOrder(Shop_Order_Model $oShopOrder)
	{
		$aoShopOrderItems = $oShopOrder->Shop_Order_Items->findAll();

		// Основные параметры заказа
		$aOrderParams = array(
			'externalId' => $oShopOrder->id,
			'firstName' => $oShopOrder->name,
			'lastName' => $oShopOrder->surname,
			'patronymic' => $oShopOrder->patronymic,
			'email' => $oShopOrder->email,
			'phone' => $oShopOrder->phone
		);

		// Список товаров, стоимость доставки и скидка
		$aItems = array();
		$deliveryCost = 0;
		$discountManualAmount = 0;

		foreach ($aoShopOrderItems as $oShopOrderItem)
		{
			$fItemPrice = Shop_Controller::instance()->round($oShopOrderItem->price + $oShopOrderItem->getTax());

			switch ($oShopOrderItem->type)
			{
				// Обычный товар или скидка
				case 0:
					if ($oShopOrderItem->price > 0)
					{
						$aItems[] = array(
							'productName' => $oShopOrderItem->name,
							'quantity' => $oShopOrderItem->quantity,
							'initialPrice' => $fItemPrice,
							'offer' => array(
								'externalId' => $oShopOrderItem->shop_item_id
							)
						);
					}
					else
					{
						$discountManualAmount += abs($fItemPrice);
					}
				break;

				// Доставка
				case 1:
					$deliveryCost += $oShopOrderItem->quantity * $fItemPrice;
				break;

				// 2 — пополнение лицевого счета, 3 — оплата с лицевого счета
				case 2:
				case 3:
					// Пропускаем
				break;
			}
		}

		$aOrderParams['items'] = $aItems;

		$aOrderParams['delivery'] = array(
			'cost' => $deliveryCost
		);

		$aOrderParams['discountManualAmount'] = $discountManualAmount;


		// Информация о доставке в комментарий
		if ($oShopOrder->Shop_Delivery->id)
		{
			$aOrderParams['customerComment'] = $oShopOrder->Shop_Delivery->name;

			$aOrderParams['customerComment'] .= "\n" . implode(', ', array_filter(array(
				$oShopOrder->Shop_Country->name,
				$oShopOrder->Shop_Country_Location->name,
				$oShopOrder->Shop_Country_Location_City->name,
				$oShopOrder->Shop_Country_Location_City_Area->name,
				$oShopOrder->address,
			)));

			if ($oShopOrder->delivery_information)
			{
				$aOrderParams['customerComment'] .= "\n" . $oShopOrder->delivery_information;
			}
		}

		// Комментарий к заказу
		if ($oShopOrder->description)
		{
			$aOrderParams['customerComment'] .= ($aOrderParams['customerComment'] ? "\n\n" : '') . $oShopOrder->description;
		}

		// Менеджер по-умолчанию
		if ($this->_defaultManager)
		{
			$aOrderParams['managerId'] = $this->_defaultManager;
		}

		// Способ заказа по-умолчнтю
		if ($this->_defaultOrderMethod)
		{
			$aOrderParams['orderType'] = $this->_defaultOrderMethod;
		}

		// Создаем заказ
		try
		{
			$response = $this->_client->request->ordersCreate($aOrderParams);
		}
		catch (\RetailCrm\Exception\CurlException $e)
		{
			self::log("Ошибка во время выгрузки заказа №{$oShopOrder->invoice} в retailCRM: " . $e->getMessage(), Core_Log::$ERROR);

			return FALSE;
		}

		if (!$response->isSuccessful())
		{
			self::log("Ошибка во время выгрузки заказа №{$oShopOrder->invoice} в retailCRM.", Core_Log::$ERROR);

			return FALSE;
		}

		self::log("Заказ №{$oShopOrder->invoice} успешно заказа в retailCRM.", Core_Log::$SUCCESS);

		return TRUE;
	}

	/**
	 * Создает заказ из подписки на товар.
	 *
	 * @param  Shop_Item_Subscription_Model  $oShopItemSubscription
	 * @return boolean
	 */
	public function createOrderFromSubscription(Shop_Item_Subscription_Model $oShopItemSubscription)
	{
		// Данные пользователя
		$oSiteuser = $oShopItemSubscription->Siteuser;

		if ($oSiteuser->id)
		{
			$aOrderParams = array(
				'firstName' => $oSiteuser->name,
				'lastName' => $oSiteuser->surname,
				'patronymic' => $oSiteuser->patronymic,
				'phone' => $oSiteuser->phone,
				'email' => $oSiteuser->email,
			);
		}
		else
		{
			$aOrderParams = array(
				'firstName' => $oShopItemSubscription->author,
				'email' => $oShopItemSubscription->email,
			);

			if (isset($oShopItemSubscription->phone))
			{
				$aOrderParams['phone'] = $oShopItemSubscription->phone;
			}
		}

		$aOrderParams['externalId'] = $oShopItemSubscription->id;

		// Статус заказа
		$aOrderParams['status'] = $this->_subscriptionOrderStatus;

		// Товар
		$oShopItem = $oShopItemSubscription->Shop_Item;

		$oShopItemController = new Shop_Item_Controller;
		$aPrices = $oShopItemController->getPrices($oShopItem);

		$aOrderParams['items'] = array(
			array(
				'productName' => $oShopItem->name,
				'quantity' => 1,
				'initialPrice' => $aPrices['price'],
				'discountManualAmount' => $aPrices['discount'],
				'offer' => array(
					'externalId' => $oShopItem->id
				)
			)
		);

		// Менеджер по-умолчанию
		if ($this->_defaultManager)
		{
			$aOrderParams['managerId'] = $this->_defaultManager;
		}

		// Создаем заказ
		try
		{
			$response = $this->_client->request->ordersCreate($aOrderParams);
		}
		catch (\RetailCrm\Exception\CurlException $e)
		{
			self::log("Ошибка во время выгрузки предзаказа в retailCRM: " . $e->getMessage(), Core_Log::$ERROR);

			return FALSE;
		}

		if (!$response->isSuccessful())
		{
			self::log("Ошибка во время выгрузки предзаказа в retailCRM.", Core_Log::$ERROR);

			return FALSE;
		}

		self::log("Предзаказ успешно выгружен в retailCRM.", Core_Log::$SUCCESS);

		return TRUE;
	}

	/**
	 * Конструктор.
	 *
	 * @param  string  $url
	 * @param  string  $apiKey
	 */
	final protected function __construct($url, $apiKey)
	{
		$this->_client = new \RetailCrm\ApiClient($url, $apiKey, \RetailCrm\ApiClient::V5);
	}
}